@@ -32,28 +32,20 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <QtCore/QVector>
 #include <QtCore/QList>
 #include <QtCore/QHash>
+#include <QtCore/QStack>
+#include <QtCore/QTimeLine>
 
 #include <KDE/KPluginFactory>
 #include <KDE/KShortcutsEditor>
 
 #include <assert.h>
-
-/** @addtogroup kwineffects */
-/** @{ */
-
-#define KWIN_EFFECT_API_MAKE_VERSION( major, minor ) (( major ) << 8 | ( minor )) 
-#define KWIN_EFFECT_API_VERSION_MAJOR 0
-#define KWIN_EFFECT_API_VERSION_MINOR 8
-#define KWIN_EFFECT_API_VERSION KWIN_EFFECT_API_MAKE_VERSION( \
-    KWIN_EFFECT_API_VERSION_MAJOR, KWIN_EFFECT_API_VERSION_MINOR )
+#include <limits.h>
 
 class KLibrary;
 class KConfigGroup;
 class KActionCollection;
 class QKeyEvent;
 
-/** @defgroup kwineffects KWin effects library */
-
 namespace KWin
 {
 
@@ -75,6 +67,114 @@ typedef QPair< Effect*, Window > InputWindowPair;
 typedef QList< EffectWindow* > EffectWindowList;
 
 
+/** @defgroup kwineffects KWin effects library
+ * KWin effects library contains necessary classes for creating new KWin
+ * compositing effects.
+ *
+ * @section creating Creating new effects
+ * This example will demonstrate the basics of creating an effect. We'll use
+ *  CoolEffect as the class name, cooleffect as internal name and
+ *  "Cool Effect" as user-visible name of the effect.
+ *
+ * This example doesn't demonstrate how to write the effect's code. For that,
+ *  see the documentation of the Effect class.
+ *
+ * @subsection creating-class CoolEffect class
+ * First you need to create CoolEffect class which has to be a subclass of
+ *  @ref KWin::Effect. In that class you can reimplement various virtual
+ *  methods to control how and where the windows are drawn.
+ *
+ * @subsection creating-macro KWIN_EFFECT macro
+ * To make KWin aware of your new effect, you first need to use the
+ *  @ref KWIN_EFFECT macro to connect your effect's class to it's internal
+ *  name. The internal name is used by KWin to identify your effect. It can be
+ *  freely chosen (although it must be a single word), must be unique and won't
+ * be shown to the user. For our example, you would use the macro like this:
+ * @code
+ * KWIN_EFFECT(cooleffect, CoolEffect)
+ * @endcode
+ *
+ * @subsection creating-buildsystem Buildsystem
+ * To build the effect, you can use the KWIN_ADD_EFFECT() cmake macro which
+ *  can be found in effects/CMakeLists.txt file in KWin's source. First
+ *  argument of the macro is the name of the library that will contain
+ *  your effect. Although not strictly required, it is usually a good idea to
+ *  use the same name as your effect's internal name there. Following arguments
+ *  to the macro are the files containing your effect's source. If our effect's
+ *  source is in cooleffect.cpp, we'd use following:
+ * @code
+ *  KWIN_ADD_EFFECT(cooleffect cooleffect.cpp)
+ * @endcode
+ *
+ * This macro takes care of compiling your effect. You'll also need to install
+ *  your effect's .desktop file, so the example CMakeLists.txt file would be
+ *  as follows:
+ * @code
+ *  KWIN_ADD_EFFECT(cooleffect cooleffect.cpp)
+ *  install( FILES cooleffect.desktop DESTINATION ${SERVICES_INSTALL_DIR}/kwin )
+ * @endcode
+ *
+ * @subsection creating-desktop Effect's .desktop file
+ * You will also need to create .desktop file to set name, description, icon
+ *  and other properties of your effect. Important fields of the .desktop file
+ *  are:
+ *  @li Name User-visible name of your effect
+ *  @li Icon Name of the icon of the effect
+ *  @li Comment Short description of the effect
+ *  @li Type must be "Service"
+ *  @li X-KDE-ServiceTypes must be "KWin/Effect"
+ *  @li X-KDE-PluginInfo-Name effect's internal name as passed to the KWIN_EFFECT macro plus "kwin4_effect_" prefix
+ *  @li X-KDE-PluginInfo-Category effect's category. Should be one of Appearance, Accessibility, Window Management, Demos, Tests, Misc
+ *  @li X-KDE-PluginInfo-EnabledByDefault whether the effect should be enabled by default (use sparingly). Default is false
+ *  @li X-KDE-Library name of the library containing the effect. This is the first argument passed to the KWIN_ADD_EFFECT macro in cmake file plus "kwin4_effect_" prefix.
+ *
+ * Example cooleffect.desktop file follows:
+ * @code
+[Desktop Entry]
+Name=Cool Effect
+Comment=The coolest effect you've ever seen
+Icon=preferences-system-windows-effect-cooleffect
+
+Type=Service
+X-KDE-ServiceTypes=KWin/Effect
+X-KDE-PluginInfo-Author=My Name
+X-KDE-PluginInfo-Email=my@email.here
+X-KDE-PluginInfo-Name=kwin4_effect_cooleffect
+X-KDE-PluginInfo-Category=Misc
+X-KDE-Library=kwin4_effect_cooleffect
+ * @endcode
+ *
+ *
+ * @section accessing Accessing windows and workspace
+ * Effects can gain access to the properties of windows and workspace via
+ *  EffectWindow and EffectsHandler classes.
+ *
+ * There is one global EffectsHandler object which you can access using the
+ *  @ref effects pointer.
+ * For each window, there is an EffectWindow object which can be used to read
+ *  window properties such as position and also to change them.
+ *
+ * For more information about this, see the documentation of the corresponding
+ *  classes.
+ *
+ * @{
+ **/
+
+#define KWIN_EFFECT_API_MAKE_VERSION( major, minor ) (( major ) << 8 | ( minor ))
+#define KWIN_EFFECT_API_VERSION_MAJOR 0
+#define KWIN_EFFECT_API_VERSION_MINOR 21
+#define KWIN_EFFECT_API_VERSION KWIN_EFFECT_API_MAKE_VERSION( \
+    KWIN_EFFECT_API_VERSION_MAJOR, KWIN_EFFECT_API_VERSION_MINOR )
+
+/**
+ * Infinite region (i.e. a special region type saying that everything needs to be painted).
+ */
+KWIN_EXPORT inline
+QRect infiniteRegion()
+    { // INT_MIN / 2 because width/height is used (INT_MIN+INT_MAX==-1)
+    return QRect( INT_MIN / 2, INT_MIN / 2, INT_MAX, INT_MAX );
+    }
+
 /**
  * @short Base class for all KWin effects
  *
@@ -376,6 +476,8 @@ class KWIN_EXPORT EffectsHandler
         virtual void setCurrentDesktop( int desktop ) = 0;
         virtual QString desktopName( int desktop ) const = 0;
         virtual int activeScreen() const = 0; // Xinerama
+        virtual int numScreens() const = 0; // Xinerama
+        virtual int screenNumber( const QPoint& pos ) const = 0; // Xinerama
         virtual QRect clientArea( clientAreaOption, int screen, int desktop ) const = 0;
         virtual QRect clientArea( clientAreaOption, const EffectWindow* c ) const = 0;
         virtual QRect clientArea( clientAreaOption, const QPoint& p, int desktop ) const = 0;
@@ -414,6 +516,7 @@ class KWIN_EXPORT EffectsHandler
          **/
         virtual void addRepaintFull() = 0;
         virtual void addRepaint( const QRect& r ) = 0;
+        virtual void addRepaint( const QRegion& r ) = 0;
         virtual void addRepaint( int x, int y, int w, int h ) = 0;
 
         CompositingType compositingType() const;
@@ -519,6 +622,7 @@ class KWIN_EXPORT EffectWindow
         virtual int height() const = 0;
         virtual QRect geometry() const = 0;
         virtual QRegion shape() const = 0;
+        virtual int screen() const = 0;
         /** @internal Do not use */
         virtual bool hasOwnShape() const = 0; // only for shadow effect, for now
         virtual QPoint pos() const = 0;
@@ -541,22 +645,94 @@ class KWIN_EXPORT EffectWindow
         virtual QString windowRole() const = 0;
         virtual const EffectWindowGroup* group() const = 0;
 
+        /**
+         * Returns whether the window is a desktop background window (the one with wallpaper).
+         * See _NET_WM_WINDOW_TYPE_DESKTOP at http://standards.freedesktop.org/wm-spec/wm-spec-latest.html .
+         */
         virtual bool isDesktop() const = 0;
+        /**
+         * Returns whether the window is a dock (i.e. a panel).
+         * See _NET_WM_WINDOW_TYPE_DOCK at http://standards.freedesktop.org/wm-spec/wm-spec-latest.html .
+         */
         virtual bool isDock() const = 0;
+        /**
+         * Returns whether the window is a standalone (detached) toolbar window.
+         * See _NET_WM_WINDOW_TYPE_TOOLBAR at http://standards.freedesktop.org/wm-spec/wm-spec-latest.html .
+         */
         virtual bool isToolbar() const = 0;
+        /**
+         * Returns whether the window is standalone menubar (AKA macmenu).
+         * This window type is a KDE extension.
+         */
         virtual bool isTopMenu() const = 0;
+        /**
+         * Returns whether the window is a torn-off menu.
+         * See _NET_WM_WINDOW_TYPE_MENU at http://standards.freedesktop.org/wm-spec/wm-spec-latest.html .
+         */
         virtual bool isMenu() const = 0;
+        /**
+         * Returns whether the window is a "normal" window, i.e. an application or any other window
+         * for which none of the specialized window types fit.
+         * See _NET_WM_WINDOW_TYPE_NORMAL at http://standards.freedesktop.org/wm-spec/wm-spec-latest.html .
+         */
         virtual bool isNormalWindow() const = 0; // normal as in 'NET::Normal or NET::Unknown non-transient'
+        /**
+         * Returns whether the window is any of special windows types (desktop, dock, splash, ...),
+         * i.e. window types that usually don't have a window frame and the user does not use window
+         * management (moving, raising,...) on them.
+         */
         virtual bool isSpecialWindow() const = 0;
+        /**
+         * Returns whether the window is a dialog window.
+         * See _NET_WM_WINDOW_TYPE_DIALOG at http://standards.freedesktop.org/wm-spec/wm-spec-latest.html .
+         */
         virtual bool isDialog() const = 0;
+        /**
+         * Returns whether the window is a splashscreen. Note that many (especially older) applications
+         * do not support marking their splash windows with this type.
+         * See _NET_WM_WINDOW_TYPE_SPLASH at http://standards.freedesktop.org/wm-spec/wm-spec-latest.html .
+         */
         virtual bool isSplash() const = 0;
+        /**
+         * Returns whether the window is a utility window, such as a tool window.
+         * See _NET_WM_WINDOW_TYPE_UTILITY at http://standards.freedesktop.org/wm-spec/wm-spec-latest.html .
+         */
         virtual bool isUtility() const = 0;
+        /**
+         * Returns whether the window is a dropdown menu (i.e. a popup directly or indirectly open
+         * from the applications menubar).
+         * See _NET_WM_WINDOW_TYPE_DROPDOWN_MENU at http://standards.freedesktop.org/wm-spec/wm-spec-latest.html .
+         */
         virtual bool isDropdownMenu() const = 0;
+        /**
+         * Returns whether the window is a popup menu (that is not a torn-off or dropdown menu).
+         * See _NET_WM_WINDOW_TYPE_POPUP_MENU at http://standards.freedesktop.org/wm-spec/wm-spec-latest.html .
+         */
         virtual bool isPopupMenu() const = 0; // a context popup, not dropdown, not torn-off
+        /**
+         * Returns whether the window is a tooltip.
+         * See _NET_WM_WINDOW_TYPE_TOOLTIP at http://standards.freedesktop.org/wm-spec/wm-spec-latest.html .
+         */
         virtual bool isTooltip() const = 0;
+        /**
+         * Returns whether the window is a window with a notification.
+         * See _NET_WM_WINDOW_TYPE_NOTIFICATION at http://standards.freedesktop.org/wm-spec/wm-spec-latest.html .
+         */
         virtual bool isNotification() const = 0;
+        /**
+         * Returns whether the window is a combobox popup.
+         * See _NET_WM_WINDOW_TYPE_COMBO at http://standards.freedesktop.org/wm-spec/wm-spec-latest.html .
+         */
         virtual bool isComboBox() const = 0;
+        /**
+         * Returns whether the window is a Drag&Drop icon.
+         * See _NET_WM_WINDOW_TYPE_DND at http://standards.freedesktop.org/wm-spec/wm-spec-latest.html .
+         */
         virtual bool isDNDIcon() const = 0;
+        /**
+         * Returns whether the window is managed by KWin (it has control over its placement and other
+         * aspects, as opposed to override-redirect windows that are entirely handled by the application).
+         */
         virtual bool isManaged() const = 0; // whether it's managed or override-redirect
 
         virtual bool isModal() const = 0;
@@ -650,10 +826,12 @@ class KWIN_EXPORT WindowQuadList
         WindowQuadList splitAtX( double x ) const;
         WindowQuadList splitAtY( double y ) const;
         WindowQuadList makeGrid( int maxquadsize ) const;
+        WindowQuadList makeRegularGrid( int xSubdivisions, int ySubdivisions ) const;
         WindowQuadList select( WindowQuadType type ) const;
         WindowQuadList filterOut( WindowQuadType type ) const;
         bool smoothNeeded() const;
         void makeArrays( float** vertices, float** texcoords ) const;
+        bool isTransformed() const;
     };
 
 class KWIN_EXPORT WindowPrePaintData
@@ -736,6 +914,226 @@ class KWIN_EXPORT ScreenPrePaintData
         QRegion paint;
     };
 
+/**
+ * @short Helper class for restricting painting area only to allowed area.
+ *
+ * This helper class helps specifying areas that should be painted, clipping
+ * out the rest. The simplest usage is creating an object on the stack
+ * and giving it the area that is allowed to be painted to. When the object
+ * is destroyed, the restriction will be removed.
+ * Note that all painting code must use paintArea() to actually perform the clipping.
+ */
+class KWIN_EXPORT PaintClipper
+    {
+    public:
+        /**
+         * Calls push().
+         */
+        PaintClipper( const QRegion& allowed_area );
+        /**
+         * Calls pop().
+         */
+        ~PaintClipper();
+        /**
+         * Allows painting only in the given area. When areas have been already
+         * specified, painting is allowed only in the intersection of all areas.
+         */
+        static void push( const QRegion& allowed_area );
+        /**
+         * Removes the given area. It must match the top item in the stack.
+         */
+        static void pop( const QRegion& allowed_area );
+        /**
+         * Returns true if any clipping should be performed.
+         */
+        static bool clip();
+        /**
+         * If clip() returns true, this function gives the resulting area in which
+         * painting is allowed. It is usually simpler to use the helper Iterator class.
+         */
+        static QRegion paintArea();
+        /**
+         * Helper class to perform the clipped painting. The usage is:
+         * @code
+         * for( PaintClipper::Iterator iterator;
+         *      !iterator.isDone();
+         *      iterator.next())
+         *     { // do the painting, possibly use iterator.boundingRect()
+         *     }
+         * @endcode
+         */
+        class KWIN_EXPORT Iterator
+            {
+            public:
+                Iterator();
+                ~Iterator();
+                bool isDone();
+                void next();
+                QRect boundingRect() const;
+            private:
+                struct Data;
+                Data* data;
+            };
+    private:
+        QRegion area;
+        static QStack< QRegion >* areas;
+    };
+
+
+/**
+ * @short Wrapper class for using timelines in KWin effects.
+ *
+ * This class provides an easy and specialised interface for
+ * effects that want a non-linear timeline. Currently, most
+ * it does is wrapping QTimeLine. In the future, this class
+ * could help using physics animations in KWin.
+ *
+ * KWin effects will usually use this feature by keeping one
+ * TimeLine per animation, for example per effect (when only
+ * one animation is done by the effect) or per windows (in
+ * the case that multiple animations can take place at the
+ * same time (such as minizing multiple windows with a short
+ * time offset. Increasing the internal time state of the
+ * TimeLine is done either by changing the 'current' time in
+ * the TimeLine, which is an int value between 0 and the
+ * duration set (defaulting to 250msec). The current time
+ * can also be changed by setting the 'progress' of the
+ * TimeLine, a double between 0.0 and 1.0. setProgress(),
+ * addProgress(), addTime(), removeTime() can all be used to
+ * manipulate the 'current' time (and at the same time
+ * progress) of the TimeLine.
+ *
+ * The internal state of the TimeLine is determined by the
+ * duration of the TimeLine, int in milliseconds, defaulting.
+ * the 'current' time and the current 'progress' are
+ * interchangeable, both determine the x-axis of a graph
+ * of a TimeLine. The value() returned represents the y-axis
+ * in this graph.
+ *
+ * m_TimeLine.setProgress(0.5) would change the 'current'
+ * time of a default TimeLine to 125 msec. m_TimeLine.value()
+ * would then return the progress value (a double between 0.0
+ * and 1.0), which can be lower than 0.5 in case the animation
+ * should start slowly, such as for the EaseInCurve.
+ * In KWin effect, the prePaintWindow() or prePaintScreen()
+ * methods have int time as one of their arguments. This int
+ * can be used to increase the 'current' time in the TimeLine.
+ * The double the is subsequently returned by value() (usually
+ * in paintWindow() or paintScreen() methods can then be used
+ * to manipulate windows, or their positions.
+ */
+class KWIN_EXPORT TimeLine
+    {
+
+    Q_ENUMS( CurveShape )
+
+    public:
+        /**
+         * The CurveShape describes the relationship between time
+         * and values. We can pass some of them through to QTimeLine
+         * but also invent our own ones.
+         */
+        enum CurveShape
+        {
+            EaseInCurve = 0,
+            EaseOutCurve,
+            EaseInOutCurve,
+            LinearCurve,
+            SineCurve
+        };
+
+        /**
+         * Creates a TimeLine and computes the progress data. The default
+         * duration can be overridden from the Effect. Usually, for larger
+         * animations you want to choose values more towards 300 milliseconds.
+         * For small animations, values around 150 milliseconds are sensible.
+         */
+        explicit TimeLine(const int duration = 250);
+
+        /**
+         * Creates a copy of the TimeLine so we can have the state copied
+         * as well.
+         */
+        TimeLine(const TimeLine &other);
+        /**
+         * Cleans up.
+         */
+        ~TimeLine();
+        /**
+         * Returns the duration of the timeline in msec.
+         */
+        int duration() const;
+        /**
+         * Set the duration of the TimeLine.
+         */
+        void setDuration(const int msec);
+        /**
+         * Returns the Value at the time set, this method will
+         * usually be used to get the progress in the paintWindow()
+         * and related methods. The value represents the y-axis' value
+         * corresponding to the current progress (or time) set by
+         * setProgress(), addProgress(), addTime(), removeTime()
+         */
+        double value() const;
+        /**
+         * Returns the Value at the time provided, this method will
+         * usually be used to get the progress in the paintWindow()
+         * and related methods, the y value of the current state x.
+         */
+        double valueForTime(const int msec) const;
+        /**
+         * Returns the current time of the TimeLine, between 0 and duration()
+         * The value returned is equivalent to the x-axis on a curve.
+         */
+        int time() const;
+        /**
+         * Returns the progress of the TimeLine, between 0.0 and 1.0.
+         * The value returned is equivalent to the y-axis on a curve.
+         */
+        double progress() const;
+        /**
+         * Increases the internal progress accounting of the timeline.
+         */
+        void addProgress(const double progress);
+        /**
+         * Increases the internal counter, this is usually done in
+         * prePaintWindow().
+         */
+        void addTime(const int msec);
+        /**
+         * Decreases the internal counter, this is usually done in
+         * prePaintWindow(). This function comes handy for reverse
+         * animations.
+         */
+        void removeTime(const int msec);
+        /**
+         * Set the time to progress * duration. This will change the
+         * internal time in the TimeLine. It's usually used in
+         * prePaintWindow() or prePaintScreen() so the value()
+         * taken in paint* is increased.
+         */
+        void setProgress(const double progress);
+        /**
+         * Set the CurveShape. The CurveShape describes the relation
+         * between the value and the time. progress is between 0 and 1
+         * It's used as input for the timeline, the x axis of the curve.
+         */
+        void setCurveShape(CurveShape curveShape);
+        /**
+         * Set the CurveShape. The CurveShape describes the relation
+         * between the value and the time.
+         */
+        //void setCurveShape(CurveShape curveShape);
+
+    private:
+        QTimeLine* m_TimeLine;
+        int m_Time;
+        double m_Progress;
+        int m_Duration;
+        CurveShape m_CurveShape;
+        //Q_DISABLE_COPY(TimeLine)
+    };
+
 /**
  * Pointer to the global EffectsHandler object.
  **/
