@@ -34,7 +34,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <kconfig.h>
 #include <klocale.h>
 #include <QApplication>
-#include <qdesktopwidget.h>
+#include <QDesktopWidget>
 #include <QAction>
 #include <stdarg.h>
 #include <kdebug.h>
@@ -61,10 +61,11 @@ TabBox::TabBox( Workspace *ws )
     , display_refcount( 0 )
     {
     setFrameStyle(QFrame::StyledPanel);
-    setFrameShadow(QFrame::Plain);
+    setFrameShadow(QFrame::Raised);
     setBackgroundRole(QPalette::Base);
-    setLineWidth(2);
-    setContentsMargins( 2, 2, 2, 2 );
+    setLineWidth(3);
+    setMidLineWidth(3);
+    setContentsMargins( 3, 3, 3, 3 );
 
     showMiniIcon = false;
 
