@@ -57,6 +57,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "scene.h"
 #include "deleted.h"
 #include "effects.h"
+#include "kdecorationfactory.h"
 
 #include <X11/extensions/shape.h>
 #include <X11/keysym.h>
@@ -133,6 +134,7 @@ Workspace::Workspace( bool restore )
     block_stacking_updates( 0 ),
     forced_global_mouse_grab( false ),
     cm_selection( NULL ),
+    compositingSuspended( false ),
     compositeRate( 0 ),
     overlay( None ),
     overlay_visible( true ),
@@ -226,7 +228,7 @@ void Workspace::init()
 //     topDock = 0L;
 //     maximizedWindowCounter = 0;
 
-    supportWindow = new QWidget;
+    supportWindow = new QWidget( NULL, Qt::X11BypassWindowManagerHint );
     XLowerWindow( display(), supportWindow->winId()); // see usage in layers.cpp
 
     XSetWindowAttributes attr;
@@ -457,7 +459,7 @@ Workspace::~Workspace()
         (*it)->release();
     delete tab_box;
     delete popupinfo;
-    delete popup;
+    discardPopup();
     XDeleteProperty(display(), rootWindow(), atoms->kwin_running);
 
     writeWindowRules();
@@ -1033,7 +1035,7 @@ void Workspace::slotReconfigure()
         updateCurrentTopMenu();
         }
 
-    if( options->useCompositing )
+    if( options->useCompositing && !compositingSuspended )
     {
         setupCompositing();
         if( effects ) // setupCompositing() may fail
@@ -1176,7 +1178,7 @@ bool Workspace::isNotManaged( const QString& title )
  */
 void Workspace::refresh()
     {
-    QWidget w;
+    QWidget w( NULL, Qt::X11BypassWindowManagerHint );
     w.setGeometry( QApplication::desktop()->geometry() );
     w.show();
     w.hide();
@@ -2373,7 +2375,7 @@ void Workspace::setupTopMenuHandling()
     connect( topmenu_selection, SIGNAL( lostOwnership()), this, SLOT( lostTopMenuSelection()));
     disconnect( topmenu_watcher, SIGNAL( lostOwner()), this, SLOT( lostTopMenuOwner()));
     managing_topmenus = true;
-    topmenu_space = new QWidget;
+    topmenu_space = new QWidget( NULL, Qt::X11BypassWindowManagerHint );
     Window stack[ 2 ];
     stack[ 0 ] = supportWindow->winId();
     stack[ 1 ] = topmenu_space->winId();
@@ -2400,6 +2402,22 @@ KDecoration* Workspace::createDecoration( KDecorationBridge* bridge )
     return mgr->createDecoration( bridge );
     }
 
+// returns a list of all colors (KDecorationDefines::ColorType) the current
+// decoration supports
+QList< int > Workspace::decorationSupportedColors() const
+    {
+    KDecorationFactory* factory = mgr->factory();
+    QList< int > ret;
+    for( Ability ab = ABILITYCOLOR_FIRST;
+         ab < ABILITYCOLOR_END;
+         ab = static_cast< Ability>( ab + 1 ))
+        {
+        if( factory->supports( ab ))
+            ret << ab;
+        }
+    return ret;
+    }
+
 QString Workspace::desktopName( int desk ) const
     {
     return QString::fromUtf8( rootInfo->desktopName( desk ) );
