@@ -43,6 +43,7 @@
 #include <kurlrequester.h>
 #include <kmimetype.h>
 #include <kservicegroup.h>
+#include <ksycoca.h>
 #include <kdebug.h>
 
 #include <assert.h>
@@ -61,6 +62,9 @@ void KConfigGroup::writeEntry( const char *pKey,
   writeEntry(pKey, int(aValue), flags);
 }
 
+namespace Kickoff
+{
+
 class AppNode
 {
 public:
@@ -86,45 +90,48 @@ public:
     QList<AppNode*> children;
 };
 
-static bool AppNodeLessThan(AppNode *n1, AppNode *n2)
-{
-    if (n1->isDir) {
-        if (n2->isDir) {
-            return n1->appName.compare(n2->appName, Qt::CaseInsensitive) < 0;
-        } else {
-            return true;
-        }
-    } else {
-        if (n2->isDir) {
-            return false;
-        } else {
-            return n1->appName.compare(n2->appName, Qt::CaseInsensitive) < 0;
-        }
-    }
-    return true;
-}
-
 class ApplicationModelPrivate
 {
 public:
     ApplicationModelPrivate(ApplicationModel *qq)
-        : q(qq), root(new AppNode()) 
-          , duplicatePolicy(ApplicationModel::ShowDuplicatesPolicy)
+        : q(qq),
+          root(new AppNode()),
+          duplicatePolicy(ApplicationModel::ShowDuplicatesPolicy),
+          sortOrder(Qt::AscendingOrder),
+          sortColumn(Qt::DisplayRole)
     {
+        systemApplications = Kickoff::systemApplicationList();
     }
+
     ~ApplicationModelPrivate()
     {
         delete root;
     }
 
+    static bool AppNodeLessThan(AppNode *n1, AppNode *n2);
     void fillNode(const QString &relPath, AppNode *node);
     static QHash<QString,QString> iconNameMap();
 
     ApplicationModel *q;
     AppNode *root;
     ApplicationModel::DuplicatePolicy duplicatePolicy;
+    Qt::SortOrder sortOrder;
+    int sortColumn;
+    QStringList systemApplications;
 };
 
+bool ApplicationModelPrivate::AppNodeLessThan(AppNode *n1, AppNode *n2)
+{
+    if (n1->isDir != n2->isDir) {
+        return n1->isDir;
+    }
+
+    const QString s1 = n1->genericName.isEmpty() ? n1->appName : n1->genericName;
+    const QString s2 = n2->genericName.isEmpty() ? n2->appName : n2->genericName;
+
+    return s1.compare(s2, Qt::CaseInsensitive) < 0;
+}
+
 void ApplicationModelPrivate::fillNode(const QString &_relPath, AppNode *node)
 {
    KServiceGroup::Ptr root = KServiceGroup::group(_relPath);
@@ -134,8 +141,8 @@ void ApplicationModelPrivate::fillNode(const QString &_relPath, AppNode *node)
 
    // application name <-> service map for detecting duplicate entries
    QHash<QString,KService::Ptr> existingServices;
-   for( KServiceGroup::List::ConstIterator it = list.begin();
-       it != list.end(); ++it)
+   for (KServiceGroup::List::ConstIterator it = list.begin();
+        it != list.end(); ++it)
    {
       QString icon;
       QString appName;
@@ -172,6 +179,12 @@ void ApplicationModelPrivate::fillNode(const QString &_relPath, AppNode *node)
                     }
                 }
          }
+         // don't duplicate applications that are configured to appear in the System tab
+         // in the Applications tab
+         if ( systemApplications.contains( service->desktopEntryName() ) ) {
+             continue;
+         }
+
          existingServices[appName] = service;
       }
       else if (p->isType(KST_KServiceGroup))
@@ -210,18 +223,18 @@ void ApplicationModelPrivate::fillNode(const QString &_relPath, AppNode *node)
       newnode->parent = node;
       node->children.append(newnode);
    }
-   qStableSort(node->children.begin(), node->children.end(), AppNodeLessThan);
+
+   qStableSort(node->children.begin(), node->children.end(), ApplicationModelPrivate::AppNodeLessThan);
 }
 
-
-
 ApplicationModel::ApplicationModel(QObject *parent)
-    : QAbstractItemModel(parent), d(new ApplicationModelPrivate(this))
+    : KickoffAbstractModel(parent), d(new ApplicationModelPrivate(this))
 {
     QDBusConnection dbus = QDBusConnection::sessionBus();
     (void)new KickoffAdaptor(this);
     QDBusConnection::sessionBus().registerObject("/kickoff", this);
     dbus.connect(QString(), "/kickoff", "org.kde.plasma", "reloadMenu", this, SLOT(slotReloadMenu()));
+    connect(KSycoca::self(), SIGNAL(databaseChanged()), this, SLOT(checkSycocaChange()));
     d->fillNode(QString(), d->root);
 }
 
@@ -373,6 +386,13 @@ void ApplicationModel::slotReloadMenu()
     reset();
 }
 
+void ApplicationModel::checkSycocaChange()
+{
+    if (KSycoca::self()->isChanged("services")) {
+        slotReloadMenu();
+    }
+}
+
 ApplicationModel::DuplicatePolicy ApplicationModel::duplicatePolicy() const
 {
     return d->duplicatePolicy;
@@ -408,4 +428,7 @@ QHash<QString,QString> ApplicationModelPrivate::iconNameMap()
     return map;
 }
 
+} // namespace Kickoff
+
+
 #include "applicationmodel.moc"
