@@ -1,5 +1,7 @@
 /*
 *   Copyright 2007 by Aaron Seigo <aseigo@kde.org>
+*   Copyright 2008 by Alexis Ménard <darktears31@gmail.com>
+*
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2,
@@ -35,18 +37,16 @@
 #include <KImageFilePreview>
 #include <KRun>
 #include <KStandardDirs>
-#include <KSvgRenderer>
 #include <KWindowSystem>
 
 #include "plasma/corona.h"
 #include "plasma/appletbrowser.h"
-#include "plasma/phase.h"
+#include "plasma/animator.h"
 #include "plasma/theme.h"
 #include "kworkspace/kworkspace.h"
 #include "knewstuff2/engine.h"
 
 #include "krunner_interface.h"
-#include "ksmserver_interface.h"
 #include "screensaver_interface.h"
 #include "ksmserver_interface.h"
 
@@ -54,19 +54,19 @@
 
 using namespace Plasma;
 
-IconLoader * DefaultDesktop::s_icons = 0;
-
 DefaultDesktop::DefaultDesktop(QObject *parent, const QVariantList &args)
     : Containment(parent, args),
       m_lockDesktopAction(0),
       m_appletBrowserAction(0),
+      m_addPanelAction(0),
       m_runCommandAction(0),
       m_lockScreenAction(0),
       m_logoutAction(0),
       m_configDialog(0),
       m_wallpaperPath(0),
       m_wallpaperPosition(0),
-      m_renderer(resolution(), 1.0)
+      m_renderer(resolution(), 1.0),
+      m_rendererToken(-1)
 {
     qRegisterMetaType<QImage>("QImage");
     qRegisterMetaType<QPersistentModelIndex>("QPersistentModelIndex");
@@ -80,17 +80,6 @@ DefaultDesktop::DefaultDesktop(QObject *parent, const QVariantList &args)
 DefaultDesktop::~DefaultDesktop()
 {
     delete m_configDialog;
-    if (s_icons && s_icons->parent() == this) {
-        // reset the static var; the s_icons objects itself is parented to us,
-        // so it'll get deleted just fine
-        s_icons = 0;
-    }
-}
-
-void DefaultDesktop::init()
-{
-    reloadConfig(true);
-    Containment::init();
 }
 
 void DefaultDesktop::nextSlide(bool skipUpdates)
@@ -100,11 +89,9 @@ void DefaultDesktop::nextSlide(bool skipUpdates)
     }
 
     if (m_slideFiles.size() > 0) {
-        // do not change to the same background (same path)
-        if (m_wallpaperPath == m_slideFiles[m_currentSlide]) {
-            if (m_slideFiles.size() == 1) {
-                return;
-            }
+        // do not change to the same background if we have a choice
+        // if there is only one background, it may be changed by someone else
+        if (m_slideFiles.size() > 1 && m_wallpaperPath == m_slideFiles[m_currentSlide]) {
             // try next one, they can't be the same (at least the same path)
             if (++m_currentSlide >= m_slideFiles.size()) {
                 m_currentSlide = 0;
@@ -123,93 +110,98 @@ QSize DefaultDesktop::resolution() const
     return QApplication::desktop()->screenGeometry(screen()).size();
 }
 
-void DefaultDesktop::constraintsUpdated(Plasma::Constraints constraints)
+void DefaultDesktop::constraintsEvent(Plasma::Constraints constraints)
 {
-    if (constraints & ScreenConstraint) {
-        if (screen() == 0 && !s_icons) {
-            s_icons = new IconLoader(this);
-        }
+    if (constraints & Plasma::StartupCompletedConstraint) {
+        qsrand(QTime(0,0,0).msecsTo(QTime::currentTime()) + id());
+        reloadConfig();
     }
 
     if (constraints & Plasma::SizeConstraint) {
-        m_renderer.setSize(resolution());
-        updateBackground();
+        m_renderer.setSize(size().toSize()); 
+
+        if (m_rendererToken != -1) {
+            // if the renderer token is still -1, then we haven't actually started up yet
+            // and there is no point in touching the renderer at this point
+            updateBackground();
+        }
     }
 
     if (constraints & Plasma::ImmutableConstraint && m_appletBrowserAction) {
         // we need to update the menu items that have already been created
-        bool locked = isImmutable();
-        m_appletBrowserAction->setVisible(!locked);
-        if (locked) {
-            m_lockDesktopAction->setIcon(KIcon("object-unlocked"));
-            m_lockDesktopAction->setText(i18n("Unlock Widgets"));
-        } else {
-            m_lockDesktopAction->setIcon(KIcon("object-locked"));
-            m_lockDesktopAction->setText(i18n("Lock Widgets"));
-        }
+        bool locked = immutability() != Mutable;
+        m_addPanelAction->setVisible(!locked);
     }
 }
 
 void DefaultDesktop::configure()
 {
     KConfigGroup cg = config();
+    KConfigGroup gcg = globalConfig();
     if (m_configDialog == 0) {
-        const QSize resolution =
+        const QSize resolution = 
             QApplication::desktop()->screenGeometry(screen()).size();
-        m_configDialog = new BackgroundDialog(resolution, cg, 0);
-        connect(m_configDialog, SIGNAL(okClicked()),
+        m_configDialog = new BackgroundDialog(resolution, cg, gcg, 0);
+        connect(m_configDialog, SIGNAL(okClicked()), 
                 this, SLOT(applyConfig()));
-        connect(m_configDialog, SIGNAL(applyClicked()),
+        connect(m_configDialog, SIGNAL(applyClicked()), 
                 this, SLOT(applyConfig()));
     }
     else {
-        m_configDialog->reloadConfig(cg);
+        m_configDialog->reloadConfig(cg, gcg);
     }
 
     m_configDialog->show();
+    KWindowSystem::setOnDesktop(m_configDialog->winId(), KWindowSystem::currentDesktop());
+    KWindowSystem::activateWindow(m_configDialog->winId());
 }
 
 void DefaultDesktop::applyConfig()
 {
     Q_ASSERT(m_configDialog);
-    m_configDialog->saveConfig(config());
+    m_configDialog->saveConfig(config(), globalConfig());
     emit configNeedsSaving();
 
     reloadConfig();
 }
 
-void DefaultDesktop::reloadConfig(bool skipUpdates)
+void DefaultDesktop::reloadConfig()
 {
-    qsrand(QTime(0,0,0).secsTo(QTime::currentTime()));
-
     KConfigGroup cg = config();
 
+    // store the state of the existing wallpaper config so we can determine later
+    // if we need to trigger updates
+    QString oldWallpaperPath(m_wallpaperPath);
+    QColor old_wallpaperColor = m_wallpaperColor;
+    int old_wallPaperPosition = m_wallpaperPosition;
+
     // If no wallpaper is set, a default will be set in updateBackground()
     // which is called as soon as constraints are updated.
-    QString oldWallpaperPath(m_wallpaperPath);
-    m_wallpaperPath = cg.readEntry("wallpaper", "");
+    m_wallpaperPath = cg.readEntry("wallpaper", QString());
+    m_backgroundMode = cg.readEntry("backgroundmode", int(BackgroundDialog::kStaticBackground));
+
+    if (m_backgroundMode != BackgroundDialog::kNoBackground &&
+        (m_wallpaperPath.isEmpty() || !KStandardDirs::exists(m_wallpaperPath)))  {
+        m_wallpaperPath = Plasma::Theme::defaultTheme()->wallpaperPath(size().toSize());
+        cg.writeEntry("wallpaper", m_wallpaperPath);
+    }
+
     if (!m_wallpaperPath.isEmpty()) {
         kDebug() << "Using configured wallpaper" << m_wallpaperPath;
     }
 
-    m_backgroundMode = cg.readEntry("backgroundmode",
-        (int) BackgroundDialog::kStaticBackground);
-
     // used in both modes, so read it no matter which mode we are in
-    int old_wallPaperPosition = m_wallpaperPosition;
-    m_wallpaperPosition = cg.readEntry("wallpaperposition", 0);
-    QColor old_wallpaperColor = m_wallpaperColor;
+    m_wallpaperPosition = cg.readEntry("wallpaperposition", int(Background::ScaleCrop));
     m_wallpaperColor = cg.readEntry("wallpapercolor", QColor(Qt::black));
 
-    if (m_backgroundMode == BackgroundDialog::kStaticBackground) {
+    if (m_backgroundMode == BackgroundDialog::kStaticBackground ||
+        m_backgroundMode == BackgroundDialog::kNoBackground) {
         m_slideshowTimer.stop();
         // Only set the wallpaper if constraints have been loaded
         // and background image has changed
-        if (!skipUpdates && screen() != -1 && (
-            oldWallpaperPath != m_wallpaperPath
-            || m_wallpaperPosition != old_wallPaperPosition
-            || m_wallpaperColor != old_wallpaperColor
-            )) {
+        if (oldWallpaperPath != m_wallpaperPath ||
+            m_wallpaperPosition != old_wallPaperPosition ||
+            m_wallpaperColor != old_wallpaperColor) {
             updateBackground();
         }
     } else {
@@ -227,7 +219,7 @@ void DefaultDesktop::reloadConfig(bool skipUpdates)
             dir.setFilter(QDir::Files | QDir::Hidden);
 
             QFileInfoList files = dir.entryInfoList();
-            foreach (QFileInfo wp, files) {
+            foreach (const QFileInfo &wp, files) {
                 int position = m_slideFiles.size() == 0 ? 0 : qrand() % m_slideFiles.size();
                 m_slideFiles.insert(position, wp.filePath());
             }
@@ -235,7 +227,7 @@ void DefaultDesktop::reloadConfig(bool skipUpdates)
             // now make it look in sub-dirs
             dir.setFilter(QDir::AllDirs | QDir::NoDotAndDotDot);
             QFileInfoList subdirs = dir.entryInfoList();
-            foreach (QFileInfo wp, subdirs) {
+            foreach (const QFileInfo &wp, subdirs) {
                 dirs.append(wp.filePath());
             }
         }
@@ -246,49 +238,52 @@ void DefaultDesktop::reloadConfig(bool skipUpdates)
             m_slideshowTimer.start();
         }
         m_currentSlide = -1;
-        nextSlide(true);
-    }
-
-    if (s_icons) {
-        s_icons->setShowIcons(cg.readEntry("showIcons",true));
-        s_icons->setGridAligned(cg.readEntry("alignToGrid",true));
+        nextSlide(false);
     }
 }
 
 void DefaultDesktop::updateBackground()
 {
-    if (m_wallpaperPath.isEmpty()) {
-        QString defaultPath = QString("EOS/contents/images/%1x%2.jpg");
-
-        QString testPath = defaultPath.arg(geometry().width()).arg(geometry().height());
-        m_wallpaperPath = KStandardDirs::locate("wallpaper", testPath);
-
-        if (m_wallpaperPath.isEmpty()) {
-            kDebug() << "Trying" << defaultPath.arg(1920).arg(1200);
-            m_wallpaperPath = KStandardDirs::locate("wallpaper", defaultPath.arg(1920).arg(1200));
-        }
-
+    if (m_wallpaperPath.isEmpty() && m_backgroundMode != BackgroundDialog::kNoBackground) {
+        m_wallpaperPath = Plasma::Theme::defaultTheme()->wallpaperPath(size().toSize());
         kDebug() << "Setting wallpaper to default" << m_wallpaperPath;
         emit configNeedsSaving();
     }
 
-    m_current_renderer_token =
+    m_rendererToken = 
         m_renderer.render(m_wallpaperPath,
                           m_wallpaperColor,
                           (Background::ResizeMethod)m_wallpaperPosition,
-                          Qt::SmoothTransformation);
+                          Qt::SmoothTransformation);                       
     suspendStartup( true ); // during KDE startup, make ksmserver until the wallpaper is ready
 }
 
 void DefaultDesktop::updateBackground(int token, const QImage &img)
 {
-    if (m_current_renderer_token == token) {
+    if (m_rendererToken == token) {
         m_bitmapBackground = QPixmap::fromImage(img);
         update();
         suspendStartup( false );
     }
 }
 
+void DefaultDesktop::addPanel()
+{
+    if (corona()) {
+        // make a panel at the top
+        Containment* panel = corona()->addContainment("panel");
+        panel->showConfigurationInterface();
+
+        panel->setScreen(screen());
+        panel->setLocation(Plasma::TopEdge);
+
+        // trigger an instant layout so we immediately have a proper geometry 
+        // rather than waiting around for the event loop
+        panel->updateConstraints(Plasma::StartupCompletedConstraint);
+        panel->flushPendingConstraintsEvents();
+    }
+}
+
 void DefaultDesktop::runCommand()
 {
     if (!KAuthorized::authorizeKAction("run_command")) {
@@ -328,37 +323,35 @@ void DefaultDesktop::lockScreen()
     }
 }
 
-QList<QAction*> DefaultDesktop::contextActions()
+QList<QAction*> DefaultDesktop::contextualActions()
 {
-    //FIXME: several items here ... probably all junior jobs =)
-    //  - pretty up the menu with separators
-    //  - should we offer "Switch User" here?
+    //TODO: should we offer "Switch User" here?
 
     if (!m_appletBrowserAction) {
-        m_appletBrowserAction = new QAction(i18n("Add Widgets..."), this);
-        connect(m_appletBrowserAction, SIGNAL(triggered(bool)), this, SIGNAL(showAddWidgets()));
-        m_appletBrowserAction->setIcon(KIcon("list-add"));
+        m_appletBrowserAction = action("add widgets");
+
+        m_addPanelAction = new QAction(i18n("Add Panel"), this);
+        connect(m_addPanelAction, SIGNAL(triggered(bool)), this, SLOT(addPanel()));
+        m_addPanelAction->setIcon(KIcon("list-add"));
 
         m_runCommandAction = new QAction(i18n("Run Command..."), this);
         connect(m_runCommandAction, SIGNAL(triggered(bool)), this, SLOT(runCommand()));
         m_runCommandAction->setIcon(KIcon("system-run"));
 
-        m_setupDesktopAction = new QAction(i18n("Configure Desktop..."), this);
+        m_setupDesktopAction = new QAction(i18n("Desktop Settings..."), this);
         m_setupDesktopAction->setIcon(KIcon("configure"));
         connect(m_setupDesktopAction, SIGNAL(triggered()), this, SLOT(configure()));
 
-        m_lockDesktopAction = new QAction(i18n("Lock Widgets"), this);
-        m_lockDesktopAction->setIcon(KIcon("object-locked"));
-        connect(m_lockDesktopAction, SIGNAL(triggered(bool)), this, SLOT(toggleDesktopImmutability()));
+        m_lockDesktopAction = action("lock widgets");
 
         m_lockScreenAction = new QAction(i18n("Lock Screen"), this);
         m_lockScreenAction->setIcon(KIcon("system-lock-screen"));
         connect(m_lockScreenAction, SIGNAL(triggered(bool)), this, SLOT(lockScreen()));
 
-        m_logoutAction = new QAction(i18n("Logout"), this);
-        m_logoutAction->setIcon(KIcon("system-log-out"));
+        m_logoutAction = new QAction(i18n("Leave..."), this);
+        m_logoutAction->setIcon(KIcon("system-shutdown"));
         connect(m_logoutAction, SIGNAL(triggered(bool)), this, SLOT(logout()));
-        constraintsUpdated(Plasma::ImmutableConstraint);
+        constraintsEvent(Plasma::ImmutableConstraint);
 
         m_separator = new QAction(this);
         m_separator->setSeparator(true);
@@ -374,18 +367,16 @@ QList<QAction*> DefaultDesktop::contextActions()
     }
 
     actions.append(m_appletBrowserAction);
+    actions.append(m_addPanelAction);
     actions.append(m_setupDesktopAction);
-
-    actions.append(m_separator);
-
-    if (s_icons && s_icons->showIcons()) {
-        //icon actions
-        actions << s_icons->contextActions();
-        actions.append(m_separator2);
+    if (screen() == -1) {
+        actions.append(action("remove"));
     }
 
     actions.append(m_lockDesktopAction);
 
+    actions.append(m_separator);
+
     if (KAuthorized::authorizeKAction("lock_screen")) {
         actions.append(m_lockScreenAction);
     }
@@ -397,28 +388,16 @@ QList<QAction*> DefaultDesktop::contextActions()
     return actions;
 }
 
-void DefaultDesktop::toggleDesktopImmutability() {
-    if (corona()) {
-        corona()->setImmutable(!corona()->isImmutable());
-    } else {
-        setImmutable(!isImmutable());
-    }
-}
-
 void DefaultDesktop::logout()
 {
     if (!KAuthorized::authorizeKAction("logout")) {
         return;
     }
-
-    QString interface("org.kde.ksmserver");
-    org::kde::KSMServerInterface smserver(interface, "/KSMServer",
-                                          QDBusConnection::sessionBus());
-    if (smserver.isValid()) {
-        smserver.logout(KWorkSpace::ShutdownConfirmDefault,
-                        KWorkSpace::ShutdownTypeDefault,
-                        KWorkSpace::ShutdownModeDefault);
-    }
+#ifndef Q_WS_WIN
+    KWorkSpace::requestShutDown(KWorkSpace::ShutdownConfirmDefault,
+                                KWorkSpace::ShutdownTypeDefault,
+                                KWorkSpace::ShutdownModeDefault);
+#endif
 }
 
 void DefaultDesktop::paintInterface(QPainter *painter,
