@@ -21,6 +21,9 @@
 
 #include <plasma/abstractrunner.h>
 
+#include <KIcon>
+#include <KService>
+
 class QAction;
 
 class WebshortcutRunner : public Plasma::AbstractRunner {
@@ -30,10 +33,8 @@ class WebshortcutRunner : public Plasma::AbstractRunner {
         WebshortcutRunner( QObject *parent, const QVariantList& args );
         ~WebshortcutRunner();
 
-        void match(Plasma::SearchContext *search);
-
-    protected:
-        void exec(Plasma::SearchMatch *action);
+        void match(Plasma::RunnerContext &context);
+        void run(const Plasma::RunnerContext &context, const Plasma::QueryMatch &match);
 
     private:
         QString getSearchQuery(const QString &query, const QString &searchWord);
@@ -42,7 +43,6 @@ class WebshortcutRunner : public Plasma::AbstractRunner {
     private:
         KService::List m_offers;
         KIcon m_icon;
-        Plasma::SearchContext::Type m_type;
 };
 
 K_EXPORT_PLASMA_RUNNER(webshortcuts, WebshortcutRunner)
